//
// Pklad 2.6 - Sky Box (Hvzdn poped)
//


// Zaka varovn kompiltoru
#pragma warning(disable:4275)
#pragma warning(disable:4251)
// Pro linker: Budeme linkovat s main,
// nikoliv WinMain, jak je na Windows zvykem
#pragma comment(linker, "/entry:\"mainCRTStartup\"")


#include <Inventor/Win/SoWin.h>
#include <Inventor/Win/viewers/SoWinFlyViewer.h>
#include <Inventor/nodes/SoBaseColor.h>
#include <Inventor/nodes/SoCamera.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoDrawStyle.h>
#include <Inventor/nodes/SoEnvironment.h>
#include <Inventor/nodes/SoLightModel.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoPointLight.h>
#include <Inventor/nodes/SoRotationXYZ.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoSphere.h>
#include <Inventor/nodes/SoTexture2.h>
#include <Inventor/nodes/SoTextureCoordinate2.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/nodes/SoIndexedTriangleStripSet.h>
#include <Inventor/engines/SoElapsedTime.h>



#define SKY_BOX_SIZE2 50   // velikost krabice (vydlen dvma)

// souadnice jednotlivch kuliek
static float skyBoxVertices[8][3] =
{
  -SKY_BOX_SIZE2, -SKY_BOX_SIZE2, -SKY_BOX_SIZE2,
   SKY_BOX_SIZE2, -SKY_BOX_SIZE2, -SKY_BOX_SIZE2,
  -SKY_BOX_SIZE2,  SKY_BOX_SIZE2, -SKY_BOX_SIZE2,
   SKY_BOX_SIZE2,  SKY_BOX_SIZE2, -SKY_BOX_SIZE2,
  -SKY_BOX_SIZE2, -SKY_BOX_SIZE2,  SKY_BOX_SIZE2,
   SKY_BOX_SIZE2, -SKY_BOX_SIZE2,  SKY_BOX_SIZE2,
  -SKY_BOX_SIZE2,  SKY_BOX_SIZE2,  SKY_BOX_SIZE2,
   SKY_BOX_SIZE2,  SKY_BOX_SIZE2,  SKY_BOX_SIZE2,
};

// indexy do skyBoxVertices pro jednotliv vertexy,
// -1 ukonuje vdy jeden triangle-strip
static int32_t skyBoxIndices[6][5] =
{
  0, 1, 2, 3, -1,  // sky00
  6, 4, 2, 0, -1,  // sky06
  5, 4, 7, 6, -1,  // sky12
  5, 7, 1, 3, -1,  // sky18
  2, 3, 6, 7, -1,  // skyN0
  4, 5, 0, 1, -1,  // skyS0
};

// indexy pro texturovac souadnice
static int32_t skyBoxTexCoordIndex[5] =
{
  0, 1, 2, 3, -1,
};


static SoSeparator* createSkyBox()
{
  int i;
  SoSeparator *root = new SoSeparator;

  // Nastav light model na BASE_COLOR, to znamen,
  // e budeme rendrovat pedsvtlenou scnu.
  // Kad objekt dostv barvu podle difuzn barvy -
  // svtla ani ostatn sloky materilu nemaj u vliv.
  SoLightModel *lmodel = new SoLightModel;
  lmodel->model.setValue(SoLightModel::BASE_COLOR);
  root->addChild(lmodel);

  // Souadnice pro rendrovan trojhelnky jsou
  // pedpipraven ve skyBoxVertices.
  SoCoordinate3 *coords = new SoCoordinate3;
  coords->point.setValues(0, 8, skyBoxVertices);
  root->addChild(coords);

  // Texturovac souadnice (v naem ppad jsou
  // spolen pro vechny strany krychle)
  SoTextureCoordinate2 *texCoord = new SoTextureCoordinate2;
  texCoord->point.set1Value(0, SbVec2f(0,0));
  texCoord->point.set1Value(1, SbVec2f(1,0));
  texCoord->point.set1Value(2, SbVec2f(0,1));
  texCoord->point.set1Value(3, SbVec2f(1,1));
  root->addChild(texCoord);

  // Textury,
  // pro kadou stranu krychle mme jednu
  // model DECAL znamen, e textura pepe pvodn barvu objektu
  SoTexture2 *textures[6];
  for (i=0; i<6; i++) {
    textures[i] = new SoTexture2;
    textures[i]->model.setValue(SoTexture2::DECAL);
  }
  textures[0]->filename.setValue("sky00.tga");
  textures[1]->filename.setValue("sky06.tga");
  textures[2]->filename.setValue("sky12.tga");
  textures[3]->filename.setValue("sky18.tga");
  textures[4]->filename.setValue("skyN0.tga");
  textures[5]->filename.setValue("skyS0.tga");

  for (i=0; i<6; i++) {
    root->addChild(textures[i]);

    // SoIndexedTriangleStripSet nm vyrendruje trojhelnky.
    // Kad poloka z coordIndex je index do pole vertex
    // z SoCoordinate3::point. Speciln hodnota -1 ukonuje
    // aktuln "triangle strip".
    SoIndexedTriangleStripSet *strip = new SoIndexedTriangleStripSet;
    strip->coordIndex.setValues(0, 5, skyBoxIndices[i]);
    strip->textureCoordIndex.setValues(0, 5, skyBoxTexCoordIndex);
    root->addChild(strip);
  }

  return root;
}



SoSeparator* createPlanet(float radius, float distance,
                          float initialAngle, SbColor color, const char *textureName)
{
  SoSeparator *root = new SoSeparator;

  // Materil planety
  if (textureName != NULL) {
    SoMaterial *material = new SoMaterial;
    material->ambientColor.setValue(SbColor(1.f, 1.f, 1.f));
    material->diffuseColor.setValue(SbColor(1.f, 1.f, 1.f));
    root->addChild(material);
  } else {
    SoMaterial *material = new SoMaterial;
    material->ambientColor.setValue(color);
    material->diffuseColor.setValue(color);
    root->addChild(material);
  }

  // Textura planety
  if (textureName != NULL) {
    SoTexture2 *texture = new SoTexture2;
    texture->filename.setValue(textureName);
    root->addChild(texture);
  }

  // Rotace (obh) kolem Slunce
  SoRotationXYZ *rotY = new SoRotationXYZ;
  rotY->axis = SoRotationXYZ::Y;
  root->addChild(rotY);

  // Pipojen engine na vstup rotanho node
  SoElapsedTime *counter = new SoElapsedTime;
  counter->speed.setValue(20.f/distance);
  rotY->angle.connectFrom(&counter->timeOut);

  // Vzdlenost od Slunce
  SoTranslation *trans = new SoTranslation;
  trans->translation.setValue(distance*float(+cos(initialAngle)), 0.f,
                              distance*float(-sin(initialAngle)));
  root->addChild(trans);

  // Model planety
  SoSphere *sphere = new SoSphere;
  sphere->radius.setValue(radius);
  root->addChild(sphere);

  return root;
}



int main(int argc, char **argv)
{
  // Inicializuj Inventor
  HWND window = SoWin::init(argv[0]);
  if (window == NULL) exit(1);

  // Vytvo koen scny
  SoSeparator *root = new SoSeparator;
  root->ref();

  // Vytvo hvzdn pozad
  root->addChild(createSkyBox());

  // Nastavme vlastnosti prosted,
  // ambientColor nechme na blou, ale intenzitu
  // nastavme na 0.25.
  SoEnvironment *envir = new SoEnvironment;
  envir->ambientIntensity.setValue(0.25f);
  root->addChild(envir);


  // Model Slunce
  // Ambientn a difuzn sloky materilu na tmav lutou,
  // emisivn na svtle lutou a spekulrn nechme nulovou (ernou)
  SoMaterial *sunMat = new SoMaterial;
  sunMat->ambientColor.setValue(0.5f, 0.5f, 0.15f);
  sunMat->diffuseColor.setValue(0.5f, 0.5f, 0.15f);
  sunMat->emissiveColor.setValue(0.9f, 0.9f, 0.3f);
  root->addChild(sunMat);

  // Koule o polomru 10
  SoSphere *sun = new SoSphere;
  sun->radius.setValue(10.f);
  root->addChild(sun);

  // Svtlo do stedu Slunce,
  // veker objekty nsledujc v grafu scny budou
  // osvtleny tmto svtlem
  SoPointLight *sunLight = new SoPointLight;
  sunLight->location.setValue(0.f, 0.f, 0.f);
  sunLight->intensity.setValue(0.75f);
  root->addChild(sunLight);


  // Model Zem
  root->addChild(createPlanet(4.0f, 45.f, 0.f,
                              SbColor(0.7f, 0.7f, 1.0f), "earth.jpg"));

  // Model Merkur
  root->addChild(createPlanet(2.0f, 20.f, float(4*M_PI/3),
                              SbColor(1.0f, 0.3f, 0.3f), NULL));

  // Model Venue
  root->addChild(createPlanet(3.0f, 30.f, float(3*M_PI/3),
                              SbColor(1.0f, 0.6f, 0.0f), "venus.jpg"));


  // Vytvo okno prohlee
  SoWinFlyViewer *viewer = new SoWinFlyViewer(window);
  viewer->setSceneGraph(root);
  viewer->setTitle("SkyBox");
  viewer->setHeadlight(FALSE);
  viewer->getCamera()->position.setValue(SbVec3f(0.f, 0.f, 40.f));
  viewer->show();

  // Zobraz okna a rozje renderovac smyku
  SoWin::show(window);
  SoWin::mainLoop();

  // Uvolni pam
  delete viewer;
  root->unref();

  return 0;
}
